'''

 Filter to implement the transformation of the INPUT DRAWABLE into an ABSTRACT PAINTING
 .......................................................................................
 
 Release 1.0   initial
 
 Flow implemented to get the final result:      ...detail the major steps
 
 1. save context
 2. prepare the new image with size as defined by the user
 3. duplicate the BG and apply a Mosaic look
 4. add some spread
 5. check where the focus of the image is
 6. split the image in four rectangle layers according to the focus
 7. manipulate each rectangle with a whirl-pinch effect
 8. combine
 9, apply an overall image manipulation still using whirl pinch
10. set this distorted mosaic layer to "difference"
11. new layer from visible, set to grain-merge
12. this is the "standard" outcome
13. if a montage of dark variations is requested:
14. - create a montage image 3*3
15. - apply a modification using a different curve on each of the nine sectors
    - if chosen by the user create an indipendent single image for each selected sector
16. - flatten and display the montage
17. if a montage of light variations is requested:
18. - create a montage 3*3
19. - apply different layer manipulations on each of the nine sectors
    - if chosen by the user create an indipendent single image for each selected sector
20. - flatten and display the montage
21. flatten the standard output image if requested
22. restore context
 
 ----------------------------------------------------------------
 COPYRIGHT NOTICE
 ----------------------------------------------------------------
 
 This program is free software you can redistribute it and/or modify
 it under the terms of the GNU General Public License as published by
 the Free Software Foundation either version 3 of the License, or
 (at your option) any later version.  
 
 This program is distributed in the hope that it will be useful,
 but WITHOUT ANY WARRANTY without even the implied warranty of
 MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 GNU General Public License for more details.
 
 You should have received a copy of the GNU General Public License
 along with this program if not, you can view the GNU General Public
 License version 3 at the web site http://www.gnu.org/licenses/gpl-3.0.html
 Alternatively you can write to the Free Software Foundation, Inc., 675 Mass
 Ave, Cambridge, MA 02139, USA.
 
 -------------------------------------------------------------------
 other info
 -------------------------------------------------------------------
  
'''
#!/usr/bin/env python
from gimpfu import *
import time
import datetime
import random
import decimal

def AbstractArt(inImage, inDrawable, widthheight, focus,
                doFlatten,
                doMontageD, singlesD, doMontageL, singlesL) :

    pdb.gimp_context_push
    currFG = pdb.gimp_context_get_foreground()
    currBG = pdb.gimp_context_get_background()

    owidth = inImage.width
    oheight = inImage.height
    type = inImage.base_type
    sourcename = inDrawable.name
    if '.' in sourcename:
        split = sourcename.split(".", 1)
        sourcename = split[0]
    
    ts = time.time()
    st = datetime.datetime.fromtimestamp(ts).strftime('%Y-%m-%d %H-%M-%S')
    
    # calculate scale factor for the new output image
    realWidthHeight = owidth+oheight      # real size of the input image
    if widthheight == 0 :   # means the user wants to have same size as input
        widthheight = realWidthHeight                                    
        scale = 1
    else :     
        optiWidthHeight = widthheight       # size to work on to get best results
        scale = decimal.Decimal(optiWidthHeight)/decimal.Decimal(realWidthHeight)      # ratio for scaling
    rwidth=int(owidth*scale)
    rheight=int(oheight*scale)
      
    pdb.gimp_image_scale (inImage, rwidth, rheight)     # note : source image will be scaled back at the end
    
    # ====================================================================================
    
    newImage = pdb.gimp_image_new (rwidth, rheight, type)                    
    newBG = pdb.gimp_layer_new_from_drawable(inDrawable, newImage)     # background layer
    newImage.add_layer (newBG, 0)
     
    newBG.mode = NORMAL_MODE
    newBG.name = "Background Image Layer"
    newBG.opacity = 100.0 
    alpha=pdb.gimp_drawable_has_alpha(newBG)
    if (alpha==False):
        pdb.gimp_layer_add_alpha (newBG)                            #  RGBA
    #pdb.gimp_hue_saturation (newBG, 0, 0, 0, +100)
        
    Mosaic = newBG.copy()
    Mosaic.mode = NORMAL_MODE
    Mosaic.name = "Mosaic Layer"
    Mosaic.opacity = 100.0 
    newImage.add_layer (Mosaic, 0)
    pdb.gimp_context_set_foreground ((0,200,200,255))   # yellow
    pdb.gimp_context_set_background ((0,200,200,255))   # yellow
    tsize = min (60, round((rwidth+rheight)/50.0,1))     
    theight = tsize/10; tspace = tsize/10; tneat = 0.00
    OCTAGONS = 2
    pdb.plug_in_mosaic (newImage, Mosaic,
                        tsize, theight, tspace, tneat,  # tile characteristics: size, height, space, neat       
                        True,                           # tile-split
                        00.0,                           # light
                        0.0,                            # colour variation     # was 0.4
                        False, True,                    # antialias, colour averaging      # was True,False
                        OCTAGONS, 0, 1)                 # octagons, surface, grout-colour (0=BG, 1=FG/BG)
    #pdb.gimp_displays_flush
    #return
    
    # added some spread
    pdb.plug_in_spread (newImage, Mosaic, tsize/5, tsize/5)
    
    #pdb.gimp_display_new (newImage)
    
    # analyze focus
    if focus == 0:  # centre
        R1X = 0; R1Y = 0; R1width = rwidth/2; R1height = rheight/2
        R2X = rwidth/2; R2Y = 0; R2width = rwidth/2; R2height = rheight/2
        R3X = 0; R3Y = rheight/2; R3width = rwidth/2; R3height = rheight/2
        R4X = rwidth/2; R4Y = rheight/2; R4width = rwidth/2; R4height = rheight/2
        RfullX = 0; RfullY = 0; Rfullwidth = rwidth; Rfullheight = rheight
        
    if focus == 1:  # top-left
        R1X = 0; R1Y = 0; R1width = rwidth/4; R1height = rheight/4;
        R2X =  rwidth/4; R2Y = 0; R2width = rwidth*3/4; R2height = rheight/4;
        R3X = 0; R3Y = rheight/4; R3width = rwidth/4; R3height = rheight*3/4;
        R4X = rwidth/4; R4Y = rheight/4; R4width = rwidth*3/4; R4height = rheight*3/4;
        RfullX = 0; RfullY = 0; Rfullwidth = rwidth/2; Rfullheight = rheight/2
        
    if focus == 2:  # top-right
        R1X = 0; R1Y = 0; R1width = rwidth*3/4; R1height = rheight/4;
        R2X =  rwidth*3/4; R2Y = 0; R2width = rwidth/4; R2height = rheight/4;
        R3X = 0; R3Y = rheight/4; R3width = rwidth*3/4; R3height = rheight*3/4;
        R4X = rwidth*3/4; R4Y = rheight/4; R4width = rwidth/4; R4height = rheight*3/4;
        RfullX = rwidth/2; RfullY = 0; Rfullwidth = rwidth/2; Rfullheight = rheight/2
        
    if focus == 3:  # bottom-left
        R1X = 0; R1Y = 0; R1width = rwidth/4; R1height = rheight*3/4;
        R2X =  rwidth/4; R2Y = 0; R2width = rwidth*3/4; R2height = rheight*3/4;
        R3X = 0; R3Y = rheight*3/4; R3width = rwidth/4; R3height = rheight/4;
        R4X = rwidth/4; R4Y = rheight*3/4; R4width = rwidth*3/4; R4height = rheight/4;
        RfullX = 0; RfullY = rheight/2; Rfullwidth = rwidth/2; Rfullheight = rheight/2
        
    if focus == 4:  # bottom-right
        R1X = 0; R1Y = 0; R1width = rwidth*3/4; R1height = rheight*3/4;
        R2X =  rwidth*3/4; R2Y = 0; R2width = rwidth/4; R2height = rheight*3/4;
        R3X = 0; R3Y = rheight*3/4; R3width = rwidth*3/4; R3height = rheight/4;
        R4X = rwidth*3/4; R4Y = rheight*3/4; R4width = rwidth/4; R4height = rheight/4;
        RfullX = rwidth/2; RfullY = rheight/2; Rfullwidth = rwidth/2; Rfullheight = rheight/2
    
    WorkCopy1 = Mosaic.copy()
    WorkCopy1.mode =  NORMAL_MODE
    WorkCopy1.name = "Mosaic Layer - tile1"
    WorkCopy1.opacity = 100.0 
    newImage.add_layer (WorkCopy1, -1)
    pdb.gimp_image_select_rectangle (newImage, CHANNEL_OP_REPLACE,
                                     R1X, R1Y, R1width, R1height)
    pdb.plug_in_whirl_pinch (newImage, WorkCopy1, random.randint(-180, -90)*1.0, 0.0, 1.0)
    pdb.gimp_selection_invert (newImage)
    pdb.gimp_layer_add_alpha (WorkCopy1)                            #  RGBA
    pdb.gimp_edit_cut (WorkCopy1)
    #pdb.gimp_displays_flush
    #return
    
    WorkCopy2 = Mosaic.copy()
    WorkCopy2.mode =  NORMAL_MODE
    WorkCopy2.name = "Mosaic Layer - tile2"
    WorkCopy2.opacity = 100.0
    newImage.add_layer (WorkCopy2, -1)
    pdb.gimp_image_select_rectangle (newImage, CHANNEL_OP_REPLACE,
                                     R2X, R2Y, R2width, R2height)
    pdb.plug_in_whirl_pinch (newImage, WorkCopy2, random.randint(90, 180)*1.0, 0.0, 1.0)
    pdb.gimp_selection_invert (newImage)
    pdb.gimp_layer_add_alpha (WorkCopy2)                            #  RGBA
    pdb.gimp_edit_cut (WorkCopy2)
    #pdb.gimp_displays_flush
    #return
    
    WorkCopy3 = Mosaic.copy()
    WorkCopy3.mode =  NORMAL_MODE
    WorkCopy3.name = "Mosaic Layer - tile3"
    WorkCopy3.opacity = 100.0
    newImage.add_layer (WorkCopy3, -1)
    pdb.gimp_image_select_rectangle (newImage, CHANNEL_OP_REPLACE,
                                     R3X, R3Y, R3width, R3height)
    pdb.plug_in_whirl_pinch (newImage, WorkCopy3, random.randint(-180, -90)*1.0, 0.0, 1.0)
    pdb.gimp_selection_invert (newImage)
    pdb.gimp_layer_add_alpha (WorkCopy3)                            #  RGBA
    pdb.gimp_edit_cut (WorkCopy3)
    #pdb.gimp_displays_flush
    #return
    
    WorkCopy4 = Mosaic.copy()
    WorkCopy4.mode =  NORMAL_MODE
    WorkCopy4.name = "Mosaic Layer - tile4"
    WorkCopy4.opacity = 100.0
    newImage.add_layer (WorkCopy4, -1)
    pdb.gimp_image_select_rectangle (newImage, CHANNEL_OP_REPLACE,
                                     R4X, R4Y, R4width, R4height)
    pdb.plug_in_whirl_pinch (newImage, WorkCopy4, random.randint(90, 180)*1.0, 0.0, 1.0)
    pdb.gimp_selection_invert (newImage)
    pdb.gimp_layer_add_alpha (WorkCopy4)                            #  RGBA
    pdb.gimp_edit_cut (WorkCopy4)
    #pdb.gimp_displays_flush
    #return
    
    # combine all four
    pdb.gimp_selection_none (newImage)
    WhirlFull = pdb.gimp_image_merge_down (newImage, WorkCopy2, CLIP_TO_BOTTOM_LAYER)
    WhirlFull = pdb.gimp_image_merge_down (newImage, WorkCopy3, CLIP_TO_BOTTOM_LAYER)
    WhirlFull = pdb.gimp_image_merge_down (newImage, WorkCopy4, CLIP_TO_BOTTOM_LAYER)
    WhirlFull.name = "Full whirl"
    WhirlFull.mode = GRAIN_MERGE_MODE
    
    #pdb.plug_in_whirl_pinch (newImage, Mosaic, 0.0, random.randint(0, 10)*0.10, random.randint(10, 20)*0.10)
    pdb.gimp_image_select_rectangle (newImage, CHANNEL_OP_REPLACE,
                                     RfullX, RfullY, Rfullwidth, Rfullheight)
    pdb.plug_in_whirl_pinch (newImage, Mosaic, -90.0, 0.0, 1.0)
    #pdb.gimp_displays_flush
    #return
    pdb.gimp_selection_none (newImage)

    WhirlPinch = pdb.gimp_image_merge_down (newImage, WhirlFull, CLIP_TO_BOTTOM_LAYER)
    WhirlPinch.name = "Full whirl and pinch"
    WhirlPinch.mode = DIFFERENCE_MODE
    pdb.plug_in_applylens (newImage, WhirlPinch, 1.75, True, False, False)
    
    VisibleS = pdb.gimp_layer_new_from_visible (newImage, newImage, "Visible")
    newImage.add_layer (VisibleS, 0)
    VisibleS.mode = GRAIN_MERGE_MODE
    VisibleS.opacity = 50.0
    pdb.plug_in_vinvert (newImage, VisibleS)
    
    OutcomeS =  pdb.gimp_layer_new_from_visible (newImage, newImage, "Outcome Standard")
    newImage.add_layer (OutcomeS, 0)
    
    # END OF STANDARD PROCEDURE ===============================================================
    
    #pdb.gimp_display_new (newImage)
    #return()
 
    if doMontageD == True:
        # ----------------------
        # prepare all 9 outcomes
        # ----------------------
        # experimental
        WhirlPinchE = WhirlPinch.copy()
        newImage.add_layer (WhirlPinchE, 0)
        pdb.plug_in_shift (newImage, WhirlPinchE, tsize, 1)
        pdb.plug_in_shift (newImage, WhirlPinchE, tsize, 0)
        # FIRST VARIATION
        pdb.gimp_item_set_visible (WhirlPinch, False)
        pdb.gimp_item_set_visible (VisibleS, False)
        pdb.gimp_item_set_visible (OutcomeS, False)
        Outcome1 = pdb.gimp_layer_new_from_visible (newImage, newImage, "Outcome 1 : DEFAULT")
        newImage.add_layer (Outcome1, 0)
        Outcome1.mode = NORMAL_MODE
        Outcome1.opacity = 100.0
    
        # SECOND VARIATION
        pdb.gimp_item_set_visible (Outcome1, False)
        Outcome2 = pdb.gimp_layer_new_from_visible (newImage, newImage, "Outcome 2 : RED CURVE 255-0-255")
        newImage.add_layer (Outcome2, 0)
        pdb.gimp_curves_spline(Outcome2, 1, 6, (0, 255, 127, 0, 255, 255))
        Outcome2.mode = NORMAL_MODE
        Outcome2.opacity = 100.0
    
        # THIRD VARIATION
        pdb.gimp_item_set_visible (Outcome2, False)
        Outcome3 = pdb.gimp_layer_new_from_visible (newImage, newImage, "Outcome 3 : GREEN CURVE 255-0-255")
        newImage.add_layer (Outcome3, 0)
        pdb.gimp_curves_spline(Outcome3, 2, 6, (0, 255, 127, 0, 255, 255))
        Outcome3.mode = NORMAL_MODE
        Outcome3.opacity = 100.0
    
        # FOURTH VARIATION
        pdb.gimp_item_set_visible (Outcome3, False)
        Outcome4 = pdb.gimp_layer_new_from_visible (newImage, newImage, "Outcome 4 : BLUE CURVE 255-0-255")
        newImage.add_layer (Outcome4, 0)
        pdb.gimp_curves_spline(Outcome4, 3, 6, (0, 255, 127, 0, 255, 255))
        Outcome4.mode = NORMAL_MODE
        Outcome4.opacity = 100.0
    
        # FIFTH VARIATION
        pdb.gimp_item_set_visible (Outcome4, False)
        Outcome5 = pdb.gimp_layer_new_from_visible (newImage, newImage, "Outcome 5 : VALUE CURVE 255-0-255")
        newImage.add_layer (Outcome5, 0)
        pdb.gimp_curves_spline(Outcome5, 0, 6, (0, 255, 127, 0, 255, 255))
        Outcome5.mode = NORMAL_MODE
        Outcome5.opacity = 100.0
    
        # SIXTH COMBO
        pdb.gimp_item_set_visible (Outcome5, False)
        Outcome6 = pdb.gimp_layer_new_from_visible (newImage, newImage, "Outcome 6 : RED CURVE 0-0-255-0-0-0-255-0-0")
        newImage.add_layer (Outcome6, 0)
        pdb.gimp_curves_spline(Outcome6, 1, 18, (0, 0, 31, 0, 63, 255, 95, 0, 127, 0, 159, 0, 191, 255, 223, 0, 255, 0))
        Outcome6.mode = NORMAL_MODE
        Outcome6.opacity = 100.0
    
        # SEVENTH COMBO
        pdb.gimp_item_set_visible (Outcome6, False)
        Outcome7 = pdb.gimp_layer_new_from_visible (newImage, newImage, "Outcome 7 : GREEN CURVE 0-0-255-0-0-0-255-0-0")
        newImage.add_layer (Outcome7, 0)
        pdb.gimp_curves_spline(Outcome7, 2, 18, (0, 0, 31, 0, 63, 255, 95, 0, 127, 0, 159, 0, 191, 255, 223, 0, 255, 0))
        Outcome7.mode = NORMAL_MODE
        Outcome7.opacity = 100.0
    
        # EIGHTH COMBO
        pdb.gimp_item_set_visible (Outcome7, False)
        Outcome8 = pdb.gimp_layer_new_from_visible (newImage, newImage, "Outcome 8 : BLUE CURVE 0-0-255-0-0-0-255-0-0")
        newImage.add_layer (Outcome8, 0)
        pdb.gimp_curves_spline(Outcome8, 3, 18, (0, 0, 31, 0, 63, 255, 95, 0, 127, 0, 159, 0, 191, 255, 223, 0, 255, 0))
        Outcome8.mode = NORMAL_MODE
        Outcome8.opacity = 100.0
    
        # NINTH COMBO
        pdb.gimp_item_set_visible (Outcome8, False)
        Outcome9 = pdb.gimp_layer_new_from_visible (newImage, newImage, "Outcome 9 : VALUE CURVE 0-0-255-0-0-0-255-0-0")
        newImage.add_layer (Outcome9, 0)
        pdb.gimp_curves_spline(Outcome9, 0, 18, (0, 0, 31, 0, 63, 255, 95, 0, 127, 0, 159, 0, 191, 255, 223, 0, 255, 0))
        Outcome9.mode = NORMAL_MODE
        Outcome9.opacity = 100.0
        

        
    # end of the 9 different outcomes preparation
    
    #pdb.gimp_displays_flush
    #return
    
    # ===========================#
    # create the montage outcome #
    # ---------------------------#
    # (if doMontageD == True: continued)
        pdb.gimp_context_set_foreground ((200,200,200,255))    # for the text on dark images
        
        QImage = pdb.gimp_image_new (rwidth*3, rheight*3, type)                    
        QBG = pdb.gimp_layer_new (QImage, rwidth*2, rheight*2, type, "BG for Montage Image", 100.0, NORMAL_MODE)     # BG layer
        QImage.add_layer (QBG, 0)
        alpha=pdb.gimp_drawable_has_alpha(QBG)
        if (alpha==False):
            pdb.gimp_layer_add_alpha (QBG)                            #  RGBA
        QBG = QImage.active_layer
        pdb.gimp_edit_fill (QBG, WHITE_FILL)
        
        # place all nine outcomes
        Q1 = pdb.gimp_layer_new (QImage, rwidth, rheight, type, "To receive 1. image", 100.0, NORMAL_MODE)      
        QImage.add_layer (Q1, 0)
        pdb.gimp_edit_copy (Outcome1)
        fs = pdb.gimp_edit_paste (Q1, True)
        pdb.gimp_floating_sel_anchor (fs)
        Text = pdb.gimp_text_layer_new (QImage, Outcome1.name, "Arial Heavy", rwidth/30, 0)
        QImage.add_layer (Text, 0)
        # if user wants it as a single image ...
        if singlesD[0] == "y":
            QI1 = pdb.gimp_image_new (rwidth, rheight, type)
            QI1BG = pdb.gimp_layer_new_from_drawable (Q1, QI1)     
            QI1.add_layer (QI1BG, 0)
            pdb.gimp_display_new (QI1)
        # end single
        Q1 = pdb.gimp_image_merge_down (QImage, Text, CLIP_TO_BOTTOM_LAYER)
        pdb.gimp_layer_translate (Q1, 0, 0)
        
        Q2 = pdb.gimp_layer_new (QImage, rwidth, rheight, type, "To receive 2. image", 100.0, NORMAL_MODE)      
        QImage.add_layer (Q2, 0)
        pdb.gimp_edit_copy (Outcome2)
        fs = pdb.gimp_edit_paste (Q2, True)
        pdb.gimp_floating_sel_anchor (fs)
        Text = pdb.gimp_text_layer_new (QImage, Outcome2.name, "Arial Heavy", rwidth/30, 0)
        QImage.add_layer (Text, 0)
        # if user wants it as a single image ...
        if singlesD[1] == "y":
            QI2 = pdb.gimp_image_new (rwidth, rheight, type)
            QI2BG = pdb.gimp_layer_new_from_drawable (Q2, QI2)     
            QI2.add_layer (QI2BG, 0)
            pdb.gimp_display_new (QI2)
        # end single
        Q2 = pdb.gimp_image_merge_down (QImage, Text, CLIP_TO_BOTTOM_LAYER)
        pdb.gimp_layer_translate (Q2, rwidth, 0)
        
        Q3 =  pdb.gimp_layer_new (QImage, rwidth, rheight, type, "To receive 3. image", 100.0, NORMAL_MODE) 
        QImage.add_layer (Q3, 0)
        pdb.gimp_edit_copy (Outcome3)
        fs = pdb.gimp_edit_paste (Q3, True)
        pdb.gimp_floating_sel_anchor (fs)
        Text = pdb.gimp_text_layer_new (QImage, Outcome3.name, "Arial Heavy", rwidth/30, 0)
        QImage.add_layer (Text, 0)
        # if user wants it as a single image ...
        if singlesD[2] == "y":
            QI3 = pdb.gimp_image_new (rwidth, rheight, type)
            QI3BG = pdb.gimp_layer_new_from_drawable (Q3, QI3)     
            QI3.add_layer (QI3BG, 0)
            pdb.gimp_display_new (QI3)
        # end single
        Q3 = pdb.gimp_image_merge_down (QImage, Text, CLIP_TO_BOTTOM_LAYER)
        pdb.gimp_layer_translate (Q3, rwidth*2, 0)
        
        Q4 = pdb.gimp_layer_new (QImage, rwidth, rheight, type, "To receive 4. image", 100.0, NORMAL_MODE)      
        QImage.add_layer (Q4, 0)
        pdb.gimp_edit_copy (Outcome4)
        fs = pdb.gimp_edit_paste (Q4, True)
        pdb.gimp_floating_sel_anchor (fs)
        Text = pdb.gimp_text_layer_new (QImage, Outcome4.name, "Arial Heavy", rwidth/30, 0)
        QImage.add_layer (Text, 0)
        # if user wants it as a single image ...
        if singlesD[3] == "y":
            QI4 = pdb.gimp_image_new (rwidth, rheight, type)
            QI4BG = pdb.gimp_layer_new_from_drawable (Q4, QI4)     
            QI4.add_layer (QI4BG, 0)
            pdb.gimp_display_new (QI4)
        # end single
        Q4 = pdb.gimp_image_merge_down (QImage, Text, CLIP_TO_BOTTOM_LAYER)
        pdb.gimp_layer_translate (Q4, 0, rheight)
        
        Q5 = pdb.gimp_layer_new (QImage, rwidth, rheight, type, "To receive 5. image", 100.0, NORMAL_MODE)      
        QImage.add_layer (Q5, 0)
        pdb.gimp_edit_copy (Outcome5)
        fs = pdb.gimp_edit_paste (Q5, True)
        pdb.gimp_floating_sel_anchor (fs)
        Text = pdb.gimp_text_layer_new (QImage, Outcome5.name, "Arial Heavy", rwidth/30, 0)
        QImage.add_layer (Text, 0)
        # if user wants it as a single image ...
        if singlesD[4] == "y":
            QI5 = pdb.gimp_image_new (rwidth, rheight, type)
            QI5BG = pdb.gimp_layer_new_from_drawable (Q5, QI5)     
            QI5.add_layer (QI5BG, 0)
            pdb.gimp_display_new (QI5)
        # end single
        Q5 = pdb.gimp_image_merge_down (QImage, Text, CLIP_TO_BOTTOM_LAYER)
        pdb.gimp_layer_translate (Q5, rwidth, rheight)
        
        Q6 = pdb.gimp_layer_new (QImage, rwidth, rheight, type, "To receive 6. image", 100.0, NORMAL_MODE)      
        QImage.add_layer (Q6, 0)
        pdb.gimp_edit_copy (Outcome6)
        fs = pdb.gimp_edit_paste (Q6, True)
        pdb.gimp_floating_sel_anchor (fs)
        Text = pdb.gimp_text_layer_new (QImage, Outcome6.name, "Arial Heavy", rwidth/30, 0)
        QImage.add_layer (Text, 0)
        # if user wants it as a single image ...
        if singlesD[5] == "y":
            QI6 = pdb.gimp_image_new (rwidth, rheight, type)
            QI6BG = pdb.gimp_layer_new_from_drawable (Q6, QI6)     
            QI6.add_layer (QI6BG, 0)
            pdb.gimp_display_new (QI6)
        # end single
        Q6 = pdb.gimp_image_merge_down (QImage, Text, CLIP_TO_BOTTOM_LAYER)
        pdb.gimp_layer_translate (Q6, rwidth*2, rheight)
        
        Q7 = pdb.gimp_layer_new (QImage, rwidth, rheight, type, "To receive 7. image", 100.0, NORMAL_MODE)      
        QImage.add_layer (Q7, 0)
        pdb.gimp_edit_copy (Outcome7)
        fs = pdb.gimp_edit_paste (Q7, True)
        pdb.gimp_floating_sel_anchor (fs)
        Text = pdb.gimp_text_layer_new (QImage, Outcome7.name, "Arial Heavy", rwidth/30, 0)
        QImage.add_layer (Text, 0)
        # if user wants it as a single image ...
        if singlesD[6] == "y":
            QI7 = pdb.gimp_image_new (rwidth, rheight, type)
            QI7BG = pdb.gimp_layer_new_from_drawable (Q7, QI7)     
            QI7.add_layer (QI7BG, 0)
            pdb.gimp_display_new (QI7)
        # end single
        Q7 = pdb.gimp_image_merge_down (QImage, Text, CLIP_TO_BOTTOM_LAYER)
        pdb.gimp_layer_translate (Q7, 0, rheight*2)
        
        Q8 = pdb.gimp_layer_new (QImage, rwidth, rheight, type, "To receive 8. image", 100.0, NORMAL_MODE)      
        QImage.add_layer (Q8, 0)
        pdb.gimp_edit_copy (Outcome8)
        fs = pdb.gimp_edit_paste (Q8, True)
        pdb.gimp_floating_sel_anchor (fs)
        Text = pdb.gimp_text_layer_new (QImage, Outcome8.name, "Arial Heavy", rwidth/30, 0)
        QImage.add_layer (Text, 0)
        # if user wants it as a single image ...
        if singlesD[7] == "y":
            QI8 = pdb.gimp_image_new (rwidth, rheight, type)
            QI8BG = pdb.gimp_layer_new_from_drawable (Q8, QI8)     
            QI8.add_layer (QI8BG, 0)
            pdb.gimp_display_new (QI8)
        # end single
        Q8 = pdb.gimp_image_merge_down (QImage, Text, CLIP_TO_BOTTOM_LAYER)
        pdb.gimp_layer_translate (Q8, rwidth, rheight*2)
        
        Q9 = pdb.gimp_layer_new (QImage, rwidth, rheight, type, "To receive 9. image", 100.0, NORMAL_MODE)      
        QImage.add_layer (Q9, 0)
        pdb.gimp_edit_copy (Outcome9)
        fs = pdb.gimp_edit_paste (Q9, True)
        pdb.gimp_floating_sel_anchor (fs)
        Text = pdb.gimp_text_layer_new (QImage, Outcome9.name, "Arial Heavy", rwidth/30, 0)
        QImage.add_layer (Text, 0)
        # if user wants it as a single image ...
        if singlesD[8] == "y":
            QI9 = pdb.gimp_image_new (rwidth, rheight, type)
            QI9BG = pdb.gimp_layer_new_from_drawable (Q9, QI9)     
            QI9.add_layer (QI9BG, 0)
            pdb.gimp_display_new (QI9)
        # end single
        Q9 = pdb.gimp_image_merge_down (QImage, Text, CLIP_TO_BOTTOM_LAYER)
        pdb.gimp_layer_translate (Q9, rwidth*2, rheight*2)
        
        # --------------------------------------------------------------------------------------------------
        
        flattened9 = pdb.gimp_image_flatten (QImage)
        flattened9.name = st+" "+sourcename+" Montage with 9 dark abstract effects"
        pdb.gimp_image_scale (QImage, rwidth, rheight)     # note : montage image scaled back to single image size
        pdb.gimp_display_new (QImage)
        pdb.gimp_image_remove_layer (newImage, Outcome9)
        pdb.gimp_image_remove_layer (newImage, Outcome8)
        pdb.gimp_image_remove_layer (newImage, Outcome7)
        pdb.gimp_image_remove_layer (newImage, Outcome6)
        pdb.gimp_image_remove_layer (newImage, Outcome5)
        pdb.gimp_image_remove_layer (newImage, Outcome4)
        pdb.gimp_image_remove_layer (newImage, Outcome3)
        pdb.gimp_image_remove_layer (newImage, Outcome2)
        pdb.gimp_image_remove_layer (newImage, Outcome1)
        pdb.gimp_image_remove_layer (newImage, WhirlPinchE)
        pdb.gimp_item_set_visible (WhirlPinch, True)        # restore

    if doMontageL == True:
        # ----------------------
        # prepare all 9 outcomes
        # ----------------------
        
        # FIRST COMBO
        pdb.gimp_item_set_visible (VisibleS, False)
        pdb.gimp_item_set_visible (OutcomeS, False)
        WhirlPinch.mode = COLOR_MODE        
        WhirlPinch.opacity = 100.0
        Visible1 = pdb.gimp_layer_new_from_visible (newImage, newImage, "Visible 1 : MULYIPLY-75 over COLOR-100")
        newImage.add_layer (Visible1, 0)
        Visible1.mode = MULTIPLY_MODE
        Visible1.opacity = 75.0
        Outcome1 = pdb.gimp_layer_new_from_visible (newImage, newImage, "Outcome 1 : MULYIPLY-75 over COLOR-100")
        newImage.add_layer (Outcome1, 0)
    
        # SECOND COMBO (exchanged)
        pdb.gimp_item_set_visible (Visible1, False)
        pdb.gimp_item_set_visible (Outcome1, False)
        WhirlPinch.mode = DIFFERENCE_MODE
        WhirlPinch.opacity = 100.0
        Visible2 = newBG.copy()
        newImage.add_layer (Visible2, 0)
        Visible2.name = "Visible 2 : DupBG on COLOR over DIFFERENCE"
        Visible2.mode = COLOR_MODE
        Visible2.opacity = 100.0
        Outcome2 = pdb.gimp_layer_new_from_visible (newImage, newImage, "Outcome 2 : DupBG on COLOR over DIFFERENCE")
        newImage.add_layer (Outcome2, 0)
        pdb.plug_in_vinvert (newImage, Outcome2)  # < < < < < < < < < <
    
        # THIRD COMBO
        pdb.gimp_item_set_visible (Visible2, False)
        pdb.gimp_item_set_visible (Outcome2, False)
        WhirlPinch.mode = VALUE_MODE
        WhirlPinch.opacity = 50.0
        Visible3 = pdb.gimp_layer_new_from_visible (newImage, newImage, "Visible 3 : SOFTLIGHT-50 over VALUE-50")
        newImage.add_layer (Visible3, 0)
        Visible3.mode = SOFTLIGHT_MODE
        Visible3.opacity = 50.0
        Outcome3 = pdb.gimp_layer_new_from_visible (newImage, newImage, "Outcome 3 : SOFTLIGHT-50 over VALUE-50")
        newImage.add_layer (Outcome3, 0)
    
        # FOURTH COMBO
        pdb.gimp_item_set_visible (Visible3, False)
        pdb.gimp_item_set_visible (Outcome3, False)
        WhirlPinch.mode = GRAIN_EXTRACT_MODE        # new
        WhirlPinch.opacity = 50.0
        Visible4 = pdb.gimp_layer_new_from_visible (newImage, newImage, "Visible 4 : GRAINMERGE-50 over GRAINEXTRACT-50")
        newImage.add_layer (Visible4, 0)
        Visible4.mode = GRAIN_MERGE_MODE
        Visible4.opacity = 50.0
        Outcome4 = pdb.gimp_layer_new_from_visible (newImage, newImage, "Outcome 4 : GRAINMERGE-50 over GRAINEXTRACT-50")
        newImage.add_layer (Outcome4, 0)
    
        # FIFTH COMBO
        pdb.gimp_item_set_visible (Visible4, False)
        pdb.gimp_item_set_visible (Outcome4, False)
        WhirlPinch.mode = HUE_MODE
        WhirlPinch.opacity = 75.0
        Visible5 = pdb.gimp_layer_new_from_visible (newImage, newImage, "Visible 5 : GRAINMERGE over HUE-75")
        newImage.add_layer (Visible5, 0)
        Visible5.mode = GRAIN_MERGE_MODE
        Visible5.opacity = 100.0
        Outcome5 = pdb.gimp_layer_new_from_visible (newImage, newImage, "Outcome 5 : GRAINMERGE over HUE-75")
        newImage.add_layer (Outcome5, 0)
    
        # SIXTH COMBO (exchanged)
        pdb.gimp_item_set_visible (Visible5, False)
        pdb.gimp_item_set_visible (Outcome5, False)
        WhirlPinch.mode = COLOR_MODE
        WhirlPinch.opacity = 75.0
        Visible6 = pdb.gimp_layer_new_from_visible (newImage, newImage, "Visible 6 : VI-SOFTLIGHT over COLOR-75")
        newImage.add_layer (Visible6, 0)
        Visible6.name = "Visible 6 : DupBG on COLOR over DIFFERENCE"
        Visible6.mode = SOFTLIGHT_MODE
        Visible6.opacity = 100.0
        pdb.plug_in_vinvert (newImage, Visible6)  
        Outcome6 = pdb.gimp_layer_new_from_visible (newImage, newImage, "Outcome 6 : VI-SOFTLIGHT over COLOR-75")
        newImage.add_layer (Outcome6, 0)
    
        # SEVENTH COMBO
        pdb.gimp_item_set_visible (Visible6, False)
        pdb.gimp_item_set_visible (Outcome6, False)
        WhirlPinch.mode = DIFFERENCE_MODE
        WhirlPinch.opacity = 100.0
        Visible7 = newBG.copy()
        newImage.add_layer (Visible7, 0)
        Visible7.name = "Visible 7 : DupBGtblur on HUE over DIFFERENCE"
        pdb.script_fu_tile_blur (newImage, Visible7, 30, True, True, 1) 
        Visible7.mode = HUE_MODE
        Visible7.opacity = 100.0
        Outcome7 = pdb.gimp_layer_new_from_visible (newImage, newImage, "Outcome 7 : DupBGtblur on HUE over DIFFERENCE")
        newImage.add_layer (Outcome7, 0)
        pdb.plug_in_vinvert (newImage, Outcome7)  # < < < < < < < < < <
    
        # EIGHTH COMBO
        pdb.gimp_item_set_visible (Visible7, False)
        pdb.gimp_item_set_visible (Outcome7, False)
        WhirlPinch.mode = DIFFERENCE_MODE
        WhirlPinch.opacity = 100.0
        Visible8 = pdb.gimp_layer_new_from_visible (newImage, newImage, "Visible 8 : DupBgBURN over VISIBLEmapPast")
        newImage.add_layer (Visible8, 0)
        Visible8.mode = NORMAL_MODE
        Visible8.opacity = 100.0
        pdb.gimp_context_set_gradient ("Pastels")
        pdb.plug_in_gradmap (newImage, Visible8)
        BGonTop8 = newBG.copy()
        newImage.add_layer (BGonTop8, 0)
        BGonTop8.mode = BURN_MODE
        Outcome8 = pdb.gimp_layer_new_from_visible (newImage, newImage, "Outcome 8 : DupBgBURN over VISIBLEmapPast")
        newImage.add_layer (Outcome8, 0)
    
        # NINTH COMBO
        pdb.gimp_item_set_visible (Visible8, False)
        pdb.gimp_item_set_visible (BGonTop8, False)
        pdb.gimp_item_set_visible (Outcome8, False)
        WhirlPinch.mode = DIFFERENCE_MODE
        WhirlPinch.opacity = 100.0
        Visible9 = pdb.gimp_layer_new_from_visible (newImage, newImage, "Visible 9 : DupBgBURN over VISIBLEmapTropical")
        newImage.add_layer (Visible9, 0)
        Visible9.mode = NORMAL_MODE
        Visible9.opacity = 100.0
        pdb.gimp_context_set_gradient ("Tropical Colors")
        pdb.plug_in_gradmap (newImage, Visible9)
        BGonTop9 = newBG.copy()
        newImage.add_layer (BGonTop9, 0)
        BGonTop9.mode = BURN_MODE
        BGonTop9.opacity = 50.0
        pdb.gimp_desaturate (BGonTop9)
        Outcome9 = pdb.gimp_layer_new_from_visible (newImage, newImage, "Outcome 9 : DupBgBURN over VISIBLEmapTropical")
        newImage.add_layer (Outcome9, 0)
        
    # end of the 9 different outcomes preparation
    
    #pdb.gimp_display_new (newImage)
    #return
    
    # ===========================#
    # create the montage outcome #
    # ---------------------------#
    # if doMontageL == True: (continued)
        pdb.gimp_context_set_foreground ((0,100,0,255))    # for the text on light images
        
        QImage = pdb.gimp_image_new (rwidth*3, rheight*3, type)                    
        QBG = pdb.gimp_layer_new (QImage, rwidth*2, rheight*2, type, "BG for Montage Image", 100.0, NORMAL_MODE)     # BG layer
        QImage.add_layer (QBG, 0)
        alpha=pdb.gimp_drawable_has_alpha(QBG)
        if (alpha==False):
            pdb.gimp_layer_add_alpha (QBG)                            #  RGBA
        QBG = QImage.active_layer
        pdb.gimp_edit_fill (QBG, WHITE_FILL)
    
        
        # place all nine outcomes
        Q1 = pdb.gimp_layer_new (QImage, rwidth, rheight, type, "To receive 1. image", 100.0, NORMAL_MODE)      
        QImage.add_layer (Q1, 0)
        pdb.gimp_edit_copy (Outcome1)
        fs = pdb.gimp_edit_paste (Q1, True)
        pdb.gimp_floating_sel_anchor (fs)
        Text = pdb.gimp_text_layer_new (QImage, Outcome1.name, "Arial Heavy", rwidth/30, 0)
        QImage.add_layer (Text, 0)
        if singlesL[0] == "y":
            QI1 = pdb.gimp_image_new (rwidth, rheight, type)
            QI1BG = pdb.gimp_layer_new_from_drawable (Q1, QI1)     
            QI1.add_layer (QI1BG, 0)
            pdb.gimp_display_new (QI1)
        # end single
        Q1 = pdb.gimp_image_merge_down (QImage, Text, CLIP_TO_BOTTOM_LAYER)
        pdb.gimp_layer_translate (Q1, 0, 0)
        
        Q2 = pdb.gimp_layer_new (QImage, rwidth, rheight, type, "To receive 2. image", 100.0, NORMAL_MODE)      
        QImage.add_layer (Q2, 0)
        pdb.gimp_edit_copy (Outcome2)
        fs = pdb.gimp_edit_paste (Q2, True)
        pdb.gimp_floating_sel_anchor (fs)
        Text = pdb.gimp_text_layer_new (QImage, Outcome2.name, "Arial Heavy", rwidth/30, 0)
        QImage.add_layer (Text, 0)
        # if user wants it as a single image ...
        if singlesL[1] == "y":
            QI2 = pdb.gimp_image_new (rwidth, rheight, type)
            QI2BG = pdb.gimp_layer_new_from_drawable (Q2, QI2)     
            QI2.add_layer (QI2BG, 0)
            pdb.gimp_display_new (QI2)
        # end single
        Q2 = pdb.gimp_image_merge_down (QImage, Text, CLIP_TO_BOTTOM_LAYER)
        pdb.gimp_layer_translate (Q2, rwidth, 0)
        
        Q3 =  pdb.gimp_layer_new (QImage, rwidth, rheight, type, "To receive 3. image", 100.0, NORMAL_MODE) 
        QImage.add_layer (Q3, 0)
        pdb.gimp_edit_copy (Outcome3)
        fs = pdb.gimp_edit_paste (Q3, True)
        pdb.gimp_floating_sel_anchor (fs)
        Text = pdb.gimp_text_layer_new (QImage, Outcome3.name, "Arial Heavy", rwidth/30, 0)
        QImage.add_layer (Text, 0)
        # if user wants it as a single image ...
        if singlesL[2] == "y":
            QI3 = pdb.gimp_image_new (rwidth, rheight, type)
            QI3BG = pdb.gimp_layer_new_from_drawable (Q3, QI3)     
            QI3.add_layer (QI3BG, 0)
            pdb.gimp_display_new (QI3)
        # end single
        Q3 = pdb.gimp_image_merge_down (QImage, Text, CLIP_TO_BOTTOM_LAYER)
        pdb.gimp_layer_translate (Q3, rwidth*2, 0)
        
        Q4 = pdb.gimp_layer_new (QImage, rwidth, rheight, type, "To receive 4. image", 100.0, NORMAL_MODE)      
        QImage.add_layer (Q4, 0)
        pdb.gimp_edit_copy (Outcome4)
        fs = pdb.gimp_edit_paste (Q4, True)
        pdb.gimp_floating_sel_anchor (fs)
        Text = pdb.gimp_text_layer_new (QImage, Outcome4.name, "Arial Heavy", rwidth/30, 0)
        QImage.add_layer (Text, 0)
        # if user wants it as a single image ...
        if singlesL[3] == "y":
            QI4 = pdb.gimp_image_new (rwidth, rheight, type)
            QI4BG = pdb.gimp_layer_new_from_drawable (Q4, QI4)     
            QI4.add_layer (QI4BG, 0)
            pdb.gimp_display_new (QI4)
        # end single
        Q4 = pdb.gimp_image_merge_down (QImage, Text, CLIP_TO_BOTTOM_LAYER)
        pdb.gimp_layer_translate (Q4, 0, rheight)
        
        Q5 = pdb.gimp_layer_new (QImage, rwidth, rheight, type, "To receive 5. image", 100.0, NORMAL_MODE)      
        QImage.add_layer (Q5, 0)
        pdb.gimp_edit_copy (Outcome5)
        fs = pdb.gimp_edit_paste (Q5, True)
        pdb.gimp_floating_sel_anchor (fs)
        Text = pdb.gimp_text_layer_new (QImage, Outcome5.name, "Arial Heavy", rwidth/30, 0)
        QImage.add_layer (Text, 0)
        # if user wants it as a single image ...
        if singlesL[4] == "y":
            QI5 = pdb.gimp_image_new (rwidth, rheight, type)
            QI5BG = pdb.gimp_layer_new_from_drawable (Q5, QI5)     
            QI5.add_layer (QI5BG, 0)
            pdb.gimp_display_new (QI5)
        # end single
        Q5 = pdb.gimp_image_merge_down (QImage, Text, CLIP_TO_BOTTOM_LAYER)
        pdb.gimp_layer_translate (Q5, rwidth, rheight)
        
        Q6 = pdb.gimp_layer_new (QImage, rwidth, rheight, type, "To receive 6. image", 100.0, NORMAL_MODE)      
        QImage.add_layer (Q6, 0)
        pdb.gimp_edit_copy (Outcome6)
        fs = pdb.gimp_edit_paste (Q6, True)
        pdb.gimp_floating_sel_anchor (fs)
        Text = pdb.gimp_text_layer_new (QImage, Outcome6.name, "Arial Heavy", rwidth/30, 0)
        QImage.add_layer (Text, 0)
        # if user wants it as a single image ...
        if singlesL[5] == "y":
            QI6 = pdb.gimp_image_new (rwidth, rheight, type)
            QI6BG = pdb.gimp_layer_new_from_drawable (Q6, QI6)     
            QI6.add_layer (QI6BG, 0)
            pdb.gimp_display_new (QI6)
        # end single
        Q6 = pdb.gimp_image_merge_down (QImage, Text, CLIP_TO_BOTTOM_LAYER)
        pdb.gimp_layer_translate (Q6, rwidth*2, rheight)
        
        Q7 = pdb.gimp_layer_new (QImage, rwidth, rheight, type, "To receive 7. image", 100.0, NORMAL_MODE)      
        QImage.add_layer (Q7, 0)
        pdb.gimp_edit_copy (Outcome7)
        fs = pdb.gimp_edit_paste (Q7, True)
        pdb.gimp_floating_sel_anchor (fs)
        Text = pdb.gimp_text_layer_new (QImage, Outcome7.name, "Arial Heavy", rwidth/30, 0)
        QImage.add_layer (Text, 0)
        # if user wants it as a single image ...
        if singlesL[6] == "y":
            QI7 = pdb.gimp_image_new (rwidth, rheight, type)
            QI7BG = pdb.gimp_layer_new_from_drawable (Q7, QI7)     
            QI7.add_layer (QI7BG, 0)
            pdb.gimp_display_new (QI7)
        # end single
        Q7 = pdb.gimp_image_merge_down (QImage, Text, CLIP_TO_BOTTOM_LAYER)
        pdb.gimp_layer_translate (Q7, 0, rheight*2)
        
        Q8 = pdb.gimp_layer_new (QImage, rwidth, rheight, type, "To receive 8. image", 100.0, NORMAL_MODE)      
        QImage.add_layer (Q8, 0)
        pdb.gimp_edit_copy (Outcome8)
        fs = pdb.gimp_edit_paste (Q8, True)
        pdb.gimp_floating_sel_anchor (fs)
        Text = pdb.gimp_text_layer_new (QImage, Outcome8.name, "Arial Heavy", rwidth/30, 0)
        QImage.add_layer (Text, 0)
        # if user wants it as a single image ...
        if singlesL[7] == "y":
            QI8 = pdb.gimp_image_new (rwidth, rheight, type)
            QI8BG = pdb.gimp_layer_new_from_drawable (Q8, QI8)     
            QI8.add_layer (QI8BG, 0)
            pdb.gimp_display_new (QI8)
        # end single
        Q8 = pdb.gimp_image_merge_down (QImage, Text, CLIP_TO_BOTTOM_LAYER)
        pdb.gimp_layer_translate (Q8, rwidth, rheight*2)
        
        Q9 = pdb.gimp_layer_new (QImage, rwidth, rheight, type, "To receive 9. image", 100.0, NORMAL_MODE)      
        QImage.add_layer (Q9, 0)
        pdb.gimp_edit_copy (Outcome9)
        fs = pdb.gimp_edit_paste (Q9, True)
        pdb.gimp_floating_sel_anchor (fs)
        Text = pdb.gimp_text_layer_new (QImage, Outcome9.name, "Arial Heavy", rwidth/30, 0)
        QImage.add_layer (Text, 0)
        # if user wants it as a single image ...
        if singlesL[8] == "y":
            QI9 = pdb.gimp_image_new (rwidth, rheight, type)
            QI9BG = pdb.gimp_layer_new_from_drawable (Q9, QI9)     
            QI9.add_layer (QI9BG, 0)
            pdb.gimp_display_new (QI9)
        # end single
        Q9 = pdb.gimp_image_merge_down (QImage, Text, CLIP_TO_BOTTOM_LAYER)
        pdb.gimp_layer_translate (Q9, rwidth*2, rheight*2)
        
        # --------------------------------------------------------------------------------------------------
        
        flattened9 = pdb.gimp_image_flatten (QImage)
        flattened9.name = st+" "+sourcename+" Montage with 9 light abstract effects"
        pdb.gimp_image_scale (QImage, rwidth, rheight)     # note : montage image scaled back to single image size
        pdb.gimp_display_new (QImage)
     
        pdb.gimp_image_remove_layer (newImage, BGonTop9)
        pdb.gimp_image_remove_layer (newImage, BGonTop8)
        pdb.gimp_image_remove_layer (newImage, Visible9)
        pdb.gimp_image_remove_layer (newImage, Visible8)
        pdb.gimp_image_remove_layer (newImage, Visible7)
        pdb.gimp_image_remove_layer (newImage, Visible6)
        pdb.gimp_image_remove_layer (newImage, Visible5)
        pdb.gimp_image_remove_layer (newImage, Visible4)
        pdb.gimp_image_remove_layer (newImage, Visible3)
        pdb.gimp_image_remove_layer (newImage, Visible2)
        pdb.gimp_image_remove_layer (newImage, Visible1)
        pdb.gimp_image_remove_layer (newImage, Outcome9)
        pdb.gimp_image_remove_layer (newImage, Outcome8)
        pdb.gimp_image_remove_layer (newImage, Outcome7)
        pdb.gimp_image_remove_layer (newImage, Outcome6)
        pdb.gimp_image_remove_layer (newImage, Outcome5)
        pdb.gimp_image_remove_layer (newImage, Outcome4)
        pdb.gimp_image_remove_layer (newImage, Outcome3)
        pdb.gimp_image_remove_layer (newImage, Outcome2)
        pdb.gimp_image_remove_layer (newImage, Outcome1)
        pdb.gimp_item_set_visible (VisibleS, True)

    # End of montage sections
    # =====================================================================================
    
    # restore standard outcome    
    pdb.gimp_item_set_visible (OutcomeS, True)
    WhirlPinch.mode = DIFFERENCE_MODE
    WhirlPinch.opacity = 100.0
    
    if doFlatten == True :
        flattenedLayer = pdb.gimp_image_merge_down (newImage, WhirlPinch, 0)
        flattenedLayer = pdb.gimp_image_merge_down (newImage, VisibleS, 0)
        flattenedLayer = pdb.gimp_image_merge_down (newImage, OutcomeS, 0)
        flattenedLayer.name = st+" "+sourcename+", as Abstract Art!"
    
    pdb.gimp_display_new (newImage)
    
    pdb.gimp_image_scale (inImage, owidth, oheight)     # note : source image scaled back  
    pdb.gimp_displays_flush
   
    # Close  
    pdb.gimp_context_pop
    pdb.gimp_context_set_foreground(currFG)
    pdb.gimp_context_set_background(currBG)
    
    if doFlatten == True :
        return(flattenedLayer)
        #return() 
    else :
        return()

# This is the plugin registration function
register(
    "AbstractArt",    
    "to tranform an image in an abstract art painting",   
    "This script transforms the input drawable into an abstract art painting.",
    "Diego", 
    "Diego Nassetti ", 
    "April 2017",
    "AbstractArt", 
    "RGB*,GRAY*", 
    [
      (PF_IMAGE, "image", "Input image", None),
      (PF_DRAWABLE, "idrawable", "Input drawable", None),
      (PF_ADJUSTMENT, "size", "Width+Height of the Out Image"+"\n (0=same as input) ",
                               3000, (0, 7500, 300)),
      (PF_OPTION, "focus", "Focus", 0, ("Centre of the image",
                                        "Top Left Quarter",
                                        "Top Right Quarter",
                                        "Bottom Left Quarter",
                                        "Bottom Right Quarter",
                                       )),
      (PF_TOGGLE, "flatten", "Flatten Image?", True),
      (PF_TOGGLE, "montaged", "Montage 9 dark variations?", True),
      (PF_STRING, "singlesd", "Get D single components? (y or n)", "nnnnnnnnn"),
      (PF_TOGGLE, "montagel", "Montage 9 light variations?", True),
      (PF_STRING, "singlesd", "Get L single components? (y or n)", "nnnnnnnnn"),
    ], 
    [
      (PF_DRAWABLE, "odrawable", "Output drawable", None),  
    ],
    AbstractArt,
    menu="<Image>/Diego/Artistic",
    )

main()
